# Texas DOT XML Request

### Overview
This is an addition to SolarNet to allow for TexasDOT to make http requests to SolarNet to gather information about a job site. This information is used in TranStar (an online tool) to display up to date information about roadside signs, radars, cameras, etc in a web format. An organization may be given an API key stored in the Mongo databse which is sent along with the request for information. 

### API Key
A new collection, OrganizationAPIKey, was added to support this feature. It has only two fields:
- _id : A string representation of the assoicated organization ID
- txDOTKey : A random value designated per organization. When a request is made, an organization and an organization key is specified. Those values are verified here before a response is generated. 

### Request
The request consists of a few different values in a URL format separated by slashes. The format is as follows:

https://api.smartzone.app/TxDOT/<ORG NAME>/<JOB SITE NAME>/<API KEY>

Below is a real example of a request for Lone Star (Not a real API Key)

https://api.smartzone.app/TxDOT/Lone%20Star/I-H10%20LoneStar%20Job%20Austin%20Cty%202/jD74Bme48vLioIY

Important: A change in Job Site name will require an update be made to the above URL. This can be done by typing the new job site name into a URL formatter and adding it in the <JOB SITE NAME> section. 

Error responses are generated if there is an issue in the request. A few examples are as follows:
- Incorrect request format. Try /TxDOT/[Organization]/[Project]/[APIKey]
- Could not find organization
- Invalid API Token

### XML Response
The XML response is a detailed summary of all the different assets within the job site. This data includes GPS locations, message board displays, radar readings, as well as snapshot information for cameras. Some of these are split into multiple sections of XML. A full sample response will be incldued at the bottom of the readme. The sections are as follows:

- Traffic Condition Data (trafficCondData)
    - This section includes detailed readings from a radar. For Doppler radars it includes averaged speed, volume, occupancy, and a time stamp of the latest read. For SideFire radars the same data is included, as well as per lane information including vehicle class counts, status, average lane speed, and occupancy.
- Network Data (networkData)
    - Network data is simply a naming standard for geographical location for each radar trailer included in the job site. It only includes a name, latitude, and longitude. 
- DMS Data (dmsData)
    - DMS data is for message boards in the job site. It includes their latitude, longitude, name, status, beacon status, message (Pipe separated for changing messages), location (roadway, direction), roadway order, as well as timestamp of last read. 
    - The location and roadway order are stored manually in the description portion of the message board. The format of this message is: TX:(IH10|West|6). This represents (Road|Direction|Order) with order being the order on the highway at which the boards are placed. If the project is on the West side of the road then the eastern most board would be 1. 
- CCTV Status Data (cctvStatusData)
    - CCTV status data includes the name, latitude, longitude, and status of the camera. It also includes a few extra static fields titled capabilities, resourceType, and lockStatus which are unchanging and included to abide by their requirements.
- CCTV Snapshot Data (cctvSnapshotData)
    - CCTV Snapshot Data includes a name, status, timestamp (Now), and a URL to request an up to date screenshot from one of the cameras. The Digital Ocean Droplets the cameras are hosted on run a node server that understands these requests (authenticated via token).

### Java Code
The changes made to SolarNet to support this API include: Organization.java, OrganizationXMLServer.java, and OrganizationAPIKey.java.

- OrganizationAPIKey.java
    - This change simply included adding the requisite Morphia class representation for the new collection. It has one important method called getTxDOTKey which queries for an API key given an organization ID. 

- OrganizationXMLServer.java
    - This class is a simple one thread HTTPS server that listens in SolarNet for requests made to api.smartzone.app. It verifies the format and queries mongo to see if the organization exists, then it queries the new API key collection to see if the org sent a valid key, then queries that org for a jobsite to verify the request is valid. If all of these pass the request is passed to Organization.java where the XML is generated on a per organization basis. 

- Organization.java
    - This is where the majority of the changes happen for this requirement. All added functions will be listed below with their purpose and how they relate to one another. 
    - public String getTxDotXML(String jobsiteTitle)
        - This is the first entry point in organization when a valid request is made. It makes sure the jobsite passed matches one that is available for this organization. When a match is not found the code returns an error message. When a match is found a few collections are created.
        - Collections are made for all Message Boards & RSTS, Cameras, and radar sensors. Each of these collections are used to create individual parts of the XML.
        -This method has a few helper methods to gather all the assets related to the jobsite making sure they are active and we are only getting the ones we want. See getRadarList, getMBList, getCameraList
    - public void getRadarXML(ArrayList<ItsSensor> radarSensors, Document doc, String jobsite)
        - This function generates two sections of the XML (See trafficCondData and networkData above)
        - For traffic condition data, depending on the radar trype, a request is made to the mongo collection for doppler or sidefire radar reading. Sidefire radar readings include a loop which analyzes data for each lane included in the sensor. Totals are taken for certain fields to calculate averages. See the example at bottom. 
        - For network data, first the method tries to gather location data directly from the radar reading. Then it attempts to gather locational data from the host asset (ie trailer), and if that fails it will use the last location reading saved in the ITS sensor. See bottom example for final XML.
    - public void getMessageBoardXML(ArrayList<MessageBoard> boards, Document doc, String jobsite)
        - This function generates the DMS section which includes details for message boards and RSTs
        - This section is more simple since we are working directly with a collection of message boards. For each message baord, an XML node is created which incldues the location requested from the board, a timestamp, a status, and the current set of messages that are schedules to play. 
        - The description field must be parsed to search for the unique string messaged above: TX:(roadway|direction|order). This is used to generate those unique fields.
    - public void getStatusXML(ArrayList<Camera> cameras, Document doc, String jobsite)
        - This function generates the cctvStatusData section for the XML. 
        - This function loops through each camera to check its status (which is available with a simple method). The GPS location data and other static details are also included here. 
    - public void getSnapshotXML(ArrayList<Camera> cameras, Document doc, String jobsite)
        - The final function which creates the cctvSnapshotData XML section
        - Each camera is looped through and its hostname (saved via Command Center) is used to generate a request URL to allow external users to request snapshots. The token is an encoded representation of the jobsite name which is stored on the Digital Ocean droplet the camera is hosted on. 

At this point the data is now ready to be sent back via OrganizationXMLServer as a string (now that Organization.java has done its job generating said string). 

### Example XML Response
# Paste this into an XML formatter 
```xml
<?xml version="1.0" encoding="UTF-8" standalone="no"?>
<Status>
  <trafficCondData>
    <net id="10001" name="I-H10 LoneStar Job Austin Cty">
      <trafficCond id="SPTT-130110" netid="10001">
        <type>Averaged</type>
        <volume>15</volume>
        <speed>67</speed>
        <occupancy>0</occupancy>
        <timestamp>2020:09:16:13:00:49</timestamp>
      </trafficCond>
      <trafficCond id="SPTT-130111" netid="10001">
        <type>Averaged</type>
        <volume>13</volume>
        <speed>70</speed>
        <occupancy>0</occupancy>
        <timestamp>2020:09:16:13:00:47</timestamp>
      </trafficCond>
      <trafficCond id="SPTT-130112" netid="10001">
        <type>Averaged</type>
        <volume>12</volume>
        <speed>64</speed>
        <occupancy>0</occupancy>
        <timestamp>2020:09:16:13:00:46</timestamp>
      </trafficCond>
      <trafficCond id="SPTT-130113" netid="10001">
        <type>Averaged</type>
        <volume>16</volume>
        <speed>70</speed>
        <occupancy>0</occupancy>
        <timestamp>2020:09:16:13:01:11</timestamp>
      </trafficCond>
      <trafficCond id="SPTT-130114" netid="10001">
        <type>Averaged</type>
        <volume>10</volume>
        <speed>57</speed>
        <occupancy>0</occupancy>
        <timestamp>2020:09:16:13:00:24</timestamp>
      </trafficCond>
      <trafficCond id="SN-MRST-210247" netid="10001">
        <type>Averaged</type>
        <volume>0</volume>
        <speed>55</speed>
        <occupancy>0</occupancy>
        <timestamp>2020:09:16:13:00:56</timestamp>
      </trafficCond>
      <trafficCond id="SN-RST-210248" netid="10001">
        <type>Averaged</type>
        <volume>0</volume>
        <speed>66</speed>
        <occupancy>0</occupancy>
        <timestamp>2020:09:16:13:00:23</timestamp>
      </trafficCond>
      <trafficCond id="SN-RST-210250" netid="10001">
        <type>Averaged</type>
        <volume>0</volume>
        <speed>63</speed>
        <occupancy>0</occupancy>
        <timestamp>2020:09:16:13:01:07</timestamp>
      </trafficCond>
      <trafficCond id="SPCT-120138" netid="10001">
        <lane-data>
          <lane-data-item>
            <detector-lane-number>1</detector-lane-number>
            <lane-status>1</lane-status>
            <lane-vehicle-count>18</lane-vehicle-count>
            <lane-vehicle-count1>10</lane-vehicle-count1>
            <lane-vehicle-count2>2</lane-vehicle-count2>
            <lane-vehicle-count3>6</lane-vehicle-count3>
            <occupancy>8</occupancy>
            <lane-vehicle-speed>68</lane-vehicle-speed>
          </lane-data-item>
          <lane-data-item>
            <detector-lane-number>2</detector-lane-number>
            <lane-status>1</lane-status>
            <lane-vehicle-count>12</lane-vehicle-count>
            <lane-vehicle-count1>0</lane-vehicle-count1>
            <lane-vehicle-count2>4</lane-vehicle-count2>
            <lane-vehicle-count3>8</lane-vehicle-count3>
            <occupancy>17</occupancy>
            <lane-vehicle-speed>68</lane-vehicle-speed>
          </lane-data-item>
        </lane-data>
        <type>Averaged</type>
        <speed>68</speed>
        <occupancy>12</occupancy>
        <volume>20</volume>
        <timestamp>2020:09:16:13:00:37</timestamp>
      </trafficCond>
      <trafficCond id="SPCT-120139" netid="10001">
        <lane-data>
          <lane-data-item>
            <detector-lane-number>1</detector-lane-number>
            <lane-status>1</lane-status>
            <lane-vehicle-count>17</lane-vehicle-count>
            <lane-vehicle-count1>12</lane-vehicle-count1>
            <lane-vehicle-count2>2</lane-vehicle-count2>
            <lane-vehicle-count3>3</lane-vehicle-count3>
            <occupancy>7</occupancy>
            <lane-vehicle-speed>71</lane-vehicle-speed>
          </lane-data-item>
          <lane-data-item>
            <detector-lane-number>2</detector-lane-number>
            <lane-status>1</lane-status>
            <lane-vehicle-count>12</lane-vehicle-count>
            <lane-vehicle-count1>10</lane-vehicle-count1>
            <lane-vehicle-count2>1</lane-vehicle-count2>
            <lane-vehicle-count3>1</lane-vehicle-count3>
            <occupancy>4</occupancy>
            <lane-vehicle-speed>70</lane-vehicle-speed>
          </lane-data-item>
        </lane-data>
        <type>Averaged</type>
        <speed>70</speed>
        <occupancy>5</occupancy>
        <volume>13</volume>
        <timestamp>2020:09:16:13:00:15</timestamp>
      </trafficCond>
      <trafficCond id="SPCT-120140" netid="10001">
        <lane-data>
          <lane-data-item>
            <detector-lane-number>1</detector-lane-number>
            <lane-status>1</lane-status>
            <lane-vehicle-count>35</lane-vehicle-count>
            <lane-vehicle-count1>24</lane-vehicle-count1>
            <lane-vehicle-count2>1</lane-vehicle-count2>
            <lane-vehicle-count3>10</lane-vehicle-count3>
            <occupancy>22</occupancy>
            <lane-vehicle-speed>66</lane-vehicle-speed>
          </lane-data-item>
          <lane-data-item>
            <detector-lane-number>2</detector-lane-number>
            <lane-status>1</lane-status>
            <lane-vehicle-count>18</lane-vehicle-count>
            <lane-vehicle-count1>8</lane-vehicle-count1>
            <lane-vehicle-count2>5</lane-vehicle-count2>
            <lane-vehicle-count3>5</lane-vehicle-count3>
            <occupancy>17</occupancy>
            <lane-vehicle-speed>73</lane-vehicle-speed>
          </lane-data-item>
        </lane-data>
        <type>Averaged</type>
        <speed>69</speed>
        <occupancy>19</occupancy>
        <volume>34</volume>
        <timestamp>2020:09:16:13:00:29</timestamp>
      </trafficCond>
      <trafficCond id="SPTT-130119" netid="10001">
        <type>Averaged</type>
        <volume>17</volume>
        <speed>57</speed>
        <occupancy>0</occupancy>
        <timestamp>2020:09:16:13:00:18</timestamp>
      </trafficCond>
      <trafficCond id="SN-RST-210249" netid="10001">
        <type>Averaged</type>
        <volume>0</volume>
        <speed>66</speed>
        <occupancy>0</occupancy>
        <timestamp>2020:09:16:13:00:35</timestamp>
      </trafficCond>
    </net>
  </trafficCondData>
  <networkData>
    <net id="10001" name="Lone Star">
      <network id="SPTT-130110" netid="10001">
        <nodeData>
          <node id="SPTT-130110" netid="10001">
            <name>SPTT-130110</name>
            <lat>29.742996025085418</lat>
            <lon>-96.32214646339419</lon>
          </node>
        </nodeData>
      </network>
      <network id="SPTT-130119" netid="10001">
        <nodeData>
          <node id="SPTT-130119" netid="10001">
            <name>SPTT-130119</name>
            <lat>29.761924695968652</lat>
            <lon>-96.1804127454758</lon>
          </node>
        </nodeData>
      </network>
      <network id="SPCT-120138" netid="10001">
        <nodeData>
          <node id="SPCT-120138" netid="10001">
            <name>SPCT-120138</name>
            <lat>29.756292915344222</lat>
            <lon>-96.23986594676963</lon>
          </node>
        </nodeData>
      </network>
      <network id="SPTT-130113" netid="10001">
        <nodeData>
          <node id="SPTT-130113" netid="10001">
            <name>SPTT-130113</name>
            <lat>29.73644778728484</lat>
            <lon>-96.35440592765802</lon>
          </node>
        </nodeData>
      </network>
      <network id="SN-RST-210248" netid="10001">
        <nodeData>
          <node id="SN-RST-210248" netid="10001">
            <name>SN-RST-210248</name>
            <lat>29.75383918062556</lat>
            <lon>-96.2569633299698</lon>
          </node>
        </nodeData>
      </network>
      <network id="SN-RST-210249" netid="10001">
        <nodeData>
          <node id="SN-RST-210249" netid="10001">
            <name>SN-RST-210249</name>
            <lat>29.746235000000016</lat>
            <lon>-96.3060980000004</lon>
          </node>
        </nodeData>
      </network>
      <network id="SPTT-130114" netid="10001">
        <nodeData>
          <node id="SPTT-130114" netid="10001">
            <name>SPTT-130114</name>
            <lat>29.761286759376524</lat>
            <lon>-96.16545910835268</lon>
          </node>
        </nodeData>
      </network>
      <network id="SPTT-130111" netid="10001">
        <nodeData>
          <node id="SPTT-130111" netid="10001">
            <name>SPTT-130111</name>
            <lat>29.739735102653484</lat>
            <lon>-96.33823006153116</lon>
          </node>
        </nodeData>
      </network>
      <network id="SN-RST-210250" netid="10001">
        <nodeData>
          <node id="SN-RST-210250" netid="10001">
            <name>SN-RST-210250</name>
            <lat>29.76261400000011</lat>
            <lon>-96.19893900000004</lon>
          </node>
        </nodeData>
      </network>
      <network id="SPTT-130112" netid="10001">
        <nodeData>
          <node id="SPTT-130112" netid="10001">
            <name>SPTT-130112</name>
            <lat>29.7657943725586</lat>
            <lon>-96.14920856952662</lon>
          </node>
        </nodeData>
      </network>
      <network id="SPCT-120139" netid="10001">
        <nodeData>
          <node id="SPCT-120139" netid="10001">
            <name>SPCT-120139</name>
            <lat>29.752954530715964</lat>
            <lon>-96.2682430744171</lon>
          </node>
        </nodeData>
      </network>
      <network id="SPCT-120140" netid="10001">
        <nodeData>
          <node id="SPCT-120140" netid="10001">
            <name>SPCT-120140</name>
            <lat>29.7511660337448</lat>
            <lon>-96.27922811508179</lon>
          </node>
        </nodeData>
      </network>
      <network id="SN-MRST-210247" netid="10001">
        <nodeData>
          <node id="SN-MRST-210247" netid="10001">
            <name>SN-MRST-210247</name>
            <lat>29.754222000000045</lat>
            <lon>-96.2571950000001</lon>
          </node>
        </nodeData>
      </network>
    </net>
  </networkData>
  <dmsData>
    <net id="10001" name="I-H10 LoneStar Job Austin Cty">
      <dms id="SN-RST-210250" netid="10001">
        <name>SN-RST-210250</name>
        <lat>29.76261400000011</lat>
        <lon>-96.19893900000004</lon>
        <status>Device Online</status>
        <beaconStatus>No Beacons</beaconStatus>
        <message>&lt;Radar&gt; | </message>
        <equipLoc>
          <roadway>IH10</roadway>
          <direction>West</direction>
        </equipLoc>
        <roadwayOrder>2</roadwayOrder>
        <timestamp>2020:09:16 13:01:15</timestamp>
      </dms>
      <dms id="EMB 6116 (new controller 826521)" netid="10001">
        <name>EMB 6116 (new controller 826521)</name>
        <lat>29.733098000000094</lat>
        <lon>-96.37080500000035</lon>
        <status>Device Online</status>
        <beaconStatus>No Beacons</beaconStatus>
        <message>ROAD WORK AHEAD | </message>
        <equipLoc>
          <roadway>IH10</roadway>
          <direction>West</direction>
        </equipLoc>
        <roadwayOrder>1</roadwayOrder>
        <timestamp>2020:09:16 13:01:15</timestamp>
      </dms>
      <dms id="SN-RST-210247" netid="10001">
        <name>SN-RST-210247</name>
        <lat>29.754222000000045</lat>
        <lon>-96.2571950000001</lon>
        <status>Device Online</status>
        <beaconStatus>No Beacons</beaconStatus>
        <message>&lt;Radar&gt;</message>
        <equipLoc>
          <roadway>IH10</roadway>
          <direction>West</direction>
        </equipLoc>
        <roadwayOrder>3</roadwayOrder>
        <timestamp>2020:09:16 13:01:15</timestamp>
      </dms>
      <dms id="EMB 6115" netid="10001">
        <name>EMB 6115</name>
        <lat>29.774905000000025</lat>
        <lon>-96.12989700000016</lon>
        <status>Device Online</status>
        <beaconStatus>No Beacons</beaconStatus>
        <message>ROAD WORK AHEAD</message>
        <equipLoc>
          <roadway>IH10</roadway>
          <direction>West</direction>
        </equipLoc>
        <roadwayOrder>6</roadwayOrder>
        <timestamp>2020:09:16 13:01:15</timestamp>
      </dms>
      <dms id="SN-RST-210248" netid="10001">
        <name>SN-RST-210248</name>
        <lat>29.75383918062556</lat>
        <lon>-96.2569633299698</lon>
        <status>Device Online</status>
        <beaconStatus>No Beacons</beaconStatus>
        <message>&lt;Radar&gt; | </message>
        <equipLoc>
          <roadway>IH10</roadway>
          <direction>West</direction>
        </equipLoc>
        <roadwayOrder>4</roadwayOrder>
        <timestamp>2020:09:16 13:01:15</timestamp>
      </dms>
      <dms id="SN-RST-210249" netid="10001">
        <name>SN-RST-210249</name>
        <lat>29.746235000000016</lat>
        <lon>-96.3060980000004</lon>
        <status>Device Online</status>
        <beaconStatus>No Beacons</beaconStatus>
        <message/>
        <equipLoc>
          <roadway>IH10</roadway>
          <direction>West</direction>
        </equipLoc>
        <roadwayOrder>5</roadwayOrder>
        <timestamp>2020:09:16 13:01:15</timestamp>
      </dms>
    </net>
  </dmsData>
  <cctvStatusData>
    <net id="10001" name="I-H10 LoneStar Job Austin Cty">
      <cctvStatus id="SPCT-120141 Camera" netid="10001">
        <name>SPCT-120141 Camera</name>
        <lat>29.753819274902337</lat>
        <lon>-96.2568638563156</lon>
        <status>Device Online</status>
        <capabilities setPreset="true"/>
        <resourceType>c2c_cam</resourceType>
        <lockStatus>Locked</lockStatus>
      </cctvStatus>
      <cctvStatus id="SPCT-120140 Camera" netid="10001">
        <name>SPCT-120140 Camera</name>
        <lat>29.7511660337448</lat>
        <lon>-96.27922811508179</lon>
        <status>Device Online</status>
        <capabilities setPreset="true"/>
        <resourceType>c2c_cam</resourceType>
        <lockStatus>Locked</lockStatus>
      </cctvStatus>
      <cctvStatus id="SPCT-120138 Camera" netid="10001">
        <name>SPCT-120138 Camera</name>
        <lat>29.756292915344222</lat>
        <lon>-96.23986594676963</lon>
        <status>Device Online</status>
        <capabilities setPreset="true"/>
        <resourceType>c2c_cam</resourceType>
        <lockStatus>Locked</lockStatus>
      </cctvStatus>
      <cctvStatus id="SPCT-120139 Camera" netid="10001">
        <name>SPCT-120139 Camera</name>
        <lat>29.752954530715964</lat>
        <lon>-96.2682430744171</lon>
        <status>Device Online</status>
        <capabilities setPreset="true"/>
        <resourceType>c2c_cam</resourceType>
        <lockStatus>Locked</lockStatus>
      </cctvStatus>
    </net>
  </cctvStatusData>
  <cctvSnapshotData>
    <net id="10001" name="I-H10 LoneStar Job Austin Cty">
      <cctvSnapshot id="SPCT-120141 Camera" netid="10001">
        <name>SPCT-120141 Camera</name>
        <status>Device Online</status>
        <timestamp>Now</timestamp>
        <snippet>https://c00047.camera.smartzone.app:9998/Get?CAPTURE?800x450?25?Token=U1BDVC0xMjAxNDEgQ2FtZXJh</snippet>
      </cctvSnapshot>
      <cctvSnapshot id="SPCT-120140 Camera" netid="10001">
        <name>SPCT-120140 Camera</name>
        <status>Device Online</status>
        <timestamp>Now</timestamp>
        <snippet>https://c00012.camera.smartzone.app:9998/Get?CAPTURE?800x450?25?Token=U1BDVC0xMjAxNDAgQ2FtZXJh</snippet>
      </cctvSnapshot>
      <cctvSnapshot id="SPCT-120138 Camera" netid="10001">
        <name>SPCT-120138 Camera</name>
        <status>Device Online</status>
        <timestamp>Now</timestamp>
        <snippet>https://c00030.camera.smartzone.app:9998/Get?CAPTURE?800x450?25?Token=U1BDVC0xMjAxMzggQ2FtZXJh</snippet>
      </cctvSnapshot>
      <cctvSnapshot id="SPCT-120139 Camera" netid="10001">
        <name>SPCT-120139 Camera</name>
        <status>Device Online</status>
        <timestamp>Now</timestamp>
        <snippet>https://c00029.camera.smartzone.app:9998/Get?CAPTURE?800x450?25?Token=U1BDVC0xMjAxMzkgQ2FtZXJh</snippet>
      </cctvSnapshot>
    </net>
  </cctvSnapshotData>
</Status>
```